@@ -10,6 +10,15 @@ MIPSolverFactory<MIPWrapper>::MIPSolverFactory() : _factoryOptions() {
   for (auto& flag : MIPWrapper::getFactoryFlags()) {
     get_global_solver_registry()->addFactoryFlag(flag, this);
   }
+
+  // Register solver without loading DLLs
+  SolverConfig sc(getId(), "<unknown version>");
+  sc.name(MIPWrapper::getName());
+  sc.mznlib(MIPWrapper::getMznLib());
+  sc.mznlibVersion(1);
+  sc.tags(MIPWrapper::getTags());
+  sc.stdFlags(MIPWrapper::getStdFlags());
+  SolverConfigs::registerBuiltinSolver(sc);
 }
 
 template <class MIPWrapper>
@@ -19,19 +28,14 @@ bool MIPSolverFactory<MIPWrapper>::processFactoryOption(int& i, std::vector<std:
 }
 
 template <class MIPWrapper>
-void MIPSolverFactory<MIPWrapper>::factoryOptionsFinished() {
+void MIPSolverFactory<MIPWrapper>::finaliseSolverConfigs(SolverConfigs& solver_configs) {
+  // Finalise solver config (needs DLLs)
+  auto& sc = solver_configs.config(getId());
   _extraFlags = MIPWrapper::getExtraFlags(_factoryOptions);
-  SolverConfig sc(getId(), MIPWrapper::getVersion(_factoryOptions, nullptr));
-  sc.name(MIPWrapper::getName());
-  sc.mznlib(MIPWrapper::getMznLib());
-  sc.mznlibVersion(1);
-  sc.supportsMzn(true);
+  sc.version(MIPWrapper::getVersion(_factoryOptions, nullptr));
   sc.description(MIPWrapper::getDescription(_factoryOptions, nullptr));
   sc.requiredFlags(MIPWrapper::getRequiredFlags(_factoryOptions));
-  sc.tags(MIPWrapper::getTags());
-  sc.stdFlags(MIPWrapper::getStdFlags());
   sc.extraFlags(_extraFlags);
-  SolverConfigs::registerBuiltinSolver(sc);
 }
 
 template <class MIPWrapper>
@@ -93,7 +97,7 @@ std::string MIPSolverFactory<MIPWrapper>::getId() {
 
 template <class MIPWrapper>
 MIPSolver::Variable MIPSolverinstance<MIPWrapper>::exprToVar(Expression* arg) {
-  if (Id* ident = arg->dynamicCast<Id>()) {
+  if (Id* ident = Expression::dynamicCast<Id>(arg)) {
     return _variableMap.get(ident->decl()->id());
   }
   return _mipWrapper->addLitVar(exprToConst(arg));
@@ -112,11 +116,11 @@ void MIPSolverinstance<MIPWrapper>::exprToVarArray(Expression* arg, std::vector<
 template <class MIPWrapper>
 std::pair<double, bool> MIPSolverinstance<MIPWrapper>::exprToConstEasy(Expression* e) {
   std::pair<double, bool> res{0.0, true};
-  if (auto* il = e->dynamicCast<IntLit>()) {
-    res.first = (static_cast<double>(il->v().toInt()));
-  } else if (auto* fl = e->dynamicCast<FloatLit>()) {
-    res.first = (fl->v().toDouble());
-  } else if (auto* bl = e->dynamicCast<BoolLit>()) {
+  if (auto* il = Expression::dynamicCast<IntLit>(e)) {
+    res.first = (static_cast<double>(IntLit::v(il).toInt()));
+  } else if (auto* fl = Expression::dynamicCast<FloatLit>(e)) {
+    res.first = (FloatLit::v(fl).toDouble());
+  } else if (auto* bl = Expression::dynamicCast<BoolLit>(e)) {
     res.first = static_cast<double>(bl->v());
   } else {
     res.second = false;
@@ -160,14 +164,14 @@ void MIPSolverinstance<MIPWrapper>::processSearchAnnotations(const Annotation& a
   std::deque<std::string> valueSelection;
 
   int nArrayAnns = 0;
-  auto priority = flattenedAnns.size();  // Variables at front get highest pri
+  auto priority = static_cast<int>(flattenedAnns.size());  // Variables at front get highest pri
   for (const auto& annExpression : flattenedAnns) {
     /// Skip expressions that are not meaningful or we cannot process
-    if (!annExpression->isa<Call>()) {
+    if (!Expression::isa<Call>(annExpression)) {
       continue;
     }
 
-    Call* annotation = annExpression->cast<Call>();
+    Call* annotation = Expression::cast<Call>(annExpression);
     const auto annotation_type = annotation->id();
     if (annotation_type != "int_search" && annotation_type != "float_search") {
       continue;
@@ -182,17 +186,17 @@ void MIPSolverinstance<MIPWrapper>::processSearchAnnotations(const Annotation& a
     /// Save the variable selection and the value selection strategies, indexed on priority.
     /// Rules are ordered by ascending priorities, i.e. rules with lower priorities are at the front
     /// so that we can index them by priority.
-    const auto cVarSel = annotation->arg(1)->cast<Id>()->str();
-    const auto cValSel = annotation->arg(2)->cast<Id>()->str();
-    variableSelection.push_front(cVarSel.c_str());
-    valueSelection.push_front(cValSel.c_str());
+    const auto cVarSel = Expression::cast<Id>(annotation->arg(1))->str();
+    const auto cValSel = Expression::cast<Id>(annotation->arg(2))->str();
+    variableSelection.emplace_front(cVarSel.c_str());
+    valueSelection.emplace_front(cValSel.c_str());
 
     ++nArrayAnns;
 
     /// Take the variables and append them with set prioirty.
     std::vector<MIPSolverinstance::VarId> annVars;
     exprToVarArray(annotation->arg(0), annVars);
-    aPri.insert(aPri.end(), annVars.size(), static_cast<int>(--priority));
+    aPri.insert(aPri.end(), annVars.size(), --priority);
     std::move(annVars.begin(), annVars.end(), std::back_inserter(vars));
   }
 
@@ -228,10 +232,10 @@ void MIPSolverinstance<MIPWrapper>::processWarmstartAnnotations(const Annotation
   int nVal = 0;
   for (ExpressionSetIter i = ann.begin(); i != ann.end(); ++i) {
     Expression* e = *i;
-    if (e->isa<Call>()) {
-      Call* c = e->cast<Call>();
+    if (Expression::isa<Call>(e)) {
+      Call* c = Expression::cast<Call>(e);
       if (c->id() == "warm_start_array" || c->id() == "seq_search") {
-        auto* anns = c->arg(0)->cast<ArrayLit>();
+        auto* anns = Expression::cast<ArrayLit>(c->arg(0));
         for (unsigned int i = 0; i < anns->size(); i++) {
           Annotation subann;
           subann.add((*anns)[i]);
@@ -254,7 +258,7 @@ void MIPSolverinstance<MIPWrapper>::processWarmstartAnnotations(const Annotation
           const auto e2c = exprToConstEasy((*alC)[i]);
           /// Check if it is not an opt int etc. and a proper variable
           if (e2c.second) {
-            if (Id* ident = (*alV)[i]->dynamicCast<Id>()) {
+            if (Id* ident = Expression::dynamicCast<Id>((*alV)[i])) {
               coefs.push_back(e2c.first);
               vars.push_back(exprToVar(ident));
             }  // else ignore
@@ -301,7 +305,7 @@ void MIPSolverinstance<MIPWrapper>::processFlatZinc() {
   VarDecl* objVd = nullptr;
 
   if (solveItem->st() != SolveI::SolveType::ST_SAT) {
-    if (Id* id = solveItem->e()->dynamicCast<Id>()) {
+    if (Id* id = Expression::dynamicCast<Id>(solveItem->e())) {
       objVd = id->decl();
     } else {
       throw InternalError("Objective must be Id");
@@ -314,9 +318,9 @@ void MIPSolverinstance<MIPWrapper>::processFlatZinc() {
       continue;
     }
     VarDecl* vd = it->e();
-    if (!vd->ann().isEmpty()) {
-      if (vd->ann().containsCall(Constants::constants().ann.output_array) ||
-          vd->ann().contains(Constants::constants().ann.output_var)) {
+    if (!Expression::ann(vd).isEmpty()) {
+      if (Expression::ann(vd).containsCall(Constants::constants().ann.output_array) ||
+          Expression::ann(vd).contains(Constants::constants().ann.output_var)) {
         _varsWithOutput.push_back(vd);
         //         std::cerr << (*vd);
         //         if ( vd->e() )
@@ -337,7 +341,7 @@ void MIPSolverinstance<MIPWrapper>::processFlatZinc() {
         ssm << "  VarDecl flags (ti, bt, st, ot): " << ti->type().ti() << ti->type().bt()
             << ti->type().st() << ti->type().ot() << ", dim == " << ti->type().dim()
             << "\nRemove the variable or add a constraint so it is redefined." << std::endl;
-        throw FlatteningError(getEnv()->envi(), ti->loc(), ssm.str());
+        throw FlatteningError(getEnv()->envi(), Expression::loc(ti), ssm.str());
       }
       double lb = 0.0;
       double ub = 1.0;  // for bool
@@ -377,9 +381,9 @@ void MIPSolverinstance<MIPWrapper>::processFlatZinc() {
       MZN_ASSERT_HARD(it->e() == id->decl());   // Assume all unified
       double obj = vd == objVd ? 1.0 : 0.0;
       auto* decl00 = follow_id_to_decl(it->e());
-      MZN_ASSERT_HARD(decl00->isa<VarDecl>());
+      MZN_ASSERT_HARD(Expression::isa<VarDecl>(decl00));
       {
-        auto* vd00 = decl00->dynamicCast<VarDecl>();
+        auto* vd00 = Expression::dynamicCast<VarDecl>(decl00);
         if (nullptr != vd00->e()) {
           // Should be a const
           auto dRHS = exprToConst(vd00->e());
@@ -435,7 +439,7 @@ void MIPSolverinstance<MIPWrapper>::processFlatZinc() {
   for (ConstraintIterator it = getEnv()->flat()->constraints().begin();
        it != getEnv()->flat()->constraints().end(); ++it) {
     if (!it->removed()) {
-      if (Call* c = it->e()->dynamicCast<Call>()) {
+      if (Call* c = Expression::dynamicCast<Call>(it->e())) {
         _constraintRegistry.post(c);
       }
     }
@@ -532,7 +536,9 @@ void MIPSolverinstance<MIPWrapper>::printStatistics() {
     StatisticsStream ss(solns2out->getOutput(), solns2out->opt.flagEncapsulateJSON);
     ss.precision(12, false);
     if (_solveType != SolveI::SolveType::ST_SAT) {
-      ss.add("objective", _mipWrapper->getObjValue());
+      if (!std::isinf(_mipWrapper->getObjValue())) {
+        ss.add("objective", _mipWrapper->getObjValue());
+      }
       if (!std::isnan(_mipWrapper->getBestBound())) {
         ss.add("objectiveBound", _mipWrapper->getBestBound());
       }
@@ -689,17 +695,18 @@ int get_mask_cons_type(const Call* call);
 /// otherwise pfx << cnt.
 inline std::string make_constraint_name(const char* pfx, int cnt,
                                         const Expression* cOrig = nullptr) {
-  Call* mznp;
   std::ostringstream ss;
-  if (nullptr != cOrig &&
-      ((mznp = cOrig->ann().getCall(Constants::constants().ann.mzn_path)) != nullptr)) {
-    assert(1 == mznp->argCount());
-    auto* strp = mznp->arg(0)->dynamicCast<StringLit>();
-    assert(strp);
-    ss << strp->v().substr(0, 255);  // Gurobi 8.1 has <=255 characters
-  } else {
-    ss << pfx << cnt;
+  if (cOrig != nullptr) {
+    auto* mznp = Expression::ann(cOrig).getCall(Constants::constants().ann.mzn_path);
+    if (mznp != nullptr) {
+      assert(1 == mznp->argCount());
+      auto* strp = Expression::dynamicCast<StringLit>(mznp->arg(0));
+      assert(strp);
+      ss << strp->v().substr(0, 255);  // Gurobi 8.1 has <=255 characters
+      return ss.str();
+    }
   }
+  ss << pfx << cnt;
   return ss.str();
 }
 
@@ -708,7 +715,7 @@ inline std::string make_constraint_name(const char* pfx, int cnt,
 template <class Idx>
 void remove_duplicates(std::vector<Idx>& rmi, std::vector<double>& rmv) {
   std::unordered_map<Idx, double> linExp;
-  for (int i = rmi.size(); i--;) {
+  for (size_t i = rmi.size(); i--;) {
     linExp[rmi[i]] += rmv[i];
   }
   if (rmi.size() == linExp.size()) {
@@ -738,10 +745,10 @@ void p_lin(SolverInstanceBase& si, const Call* call, typename MIPWrapper::LinCon
   FloatVal fres;
 
   double rhs;
-  if (call->arg(2)->type().isint()) {
+  if (Expression::type(call->arg(2)).isint()) {
     ires = eval_int(_env.envi(), call->arg(2));
     rhs = static_cast<double>(ires.toInt());
-  } else if (call->arg(2)->type().isfloat()) {
+  } else if (Expression::type(call->arg(2)).isfloat()) {
     fres = eval_float(_env.envi(), call->arg(2));
     rhs = fres.toDouble();
   } else {
@@ -755,7 +762,7 @@ void p_lin(SolverInstanceBase& si, const Call* call, typename MIPWrapper::LinCon
   vars.reserve(alV->size());
   for (unsigned int i = 0; i < alV->size(); i++) {
     const double dCoef = gi.exprToConst((*alC)[i]);
-    if (Id* ident = (*alV)[i]->dynamicCast<Id>()) {
+    if (Id* ident = Expression::dynamicCast<Id>((*alV)[i])) {
       coefs.push_back(dCoef);
       vars.push_back(gi.exprToVar(ident));
     } else {
@@ -809,13 +816,13 @@ void p_non_lin(SolverInstanceBase& si, const Call* call, typename MIPWrapper::Li
   std::vector<double> coefs;
   std::vector<MIPSolver::Variable> vars;
   double rhs = 0.0;
-  if (call->arg(0)->isa<Id>()) {
+  if (Expression::isa<Id>(call->arg(0))) {
     coefs.push_back(1.0);
     vars.push_back(gi.exprToVar(call->arg(0)));
   } else {
     rhs -= gi.exprToConst(call->arg(0));
   }
-  if (call->arg(1)->isa<Id>()) {
+  if (Expression::isa<Id>(call->arg(1))) {
     coefs.push_back(-1.0);
     vars.push_back(gi.exprToVar(call->arg(1)));
   } else {
@@ -860,13 +867,13 @@ void p_indicator_le0_if0(SolverInstanceBase& si, const Call* call) {
   double val2;
   MIPSolver::Variable var1;
   MIPSolver::Variable var2;
-  if (call->arg(0)->isa<Id>()) {
+  if (Expression::isa<Id>(call->arg(0))) {
     var1 = gi.exprToVar(call->arg(0));
   } else {
     f1const = 1;
     val1 = gi.exprToConst(call->arg(0));
   }
-  if (call->arg(1)->isa<Id>()) {
+  if (Expression::isa<Id>(call->arg(1))) {
     var2 = gi.exprToVar(call->arg(1));
   } else {
     f2const = 1;
@@ -915,7 +922,7 @@ void p_indicator_eq_if1(SolverInstanceBase& si, const Call* call) {
   MIPSolver::Variable var1;
   MIPSolver::Variable var2;
   MIPSolver::Variable varB;
-  if (call->arg(0)->isa<Id>()) {
+  if (Expression::isa<Id>(call->arg(0))) {
     var1 = gi.exprToVar(call->arg(0));
     coefs.push_back(1.0);
     vars.push_back(var1);
@@ -924,7 +931,7 @@ void p_indicator_eq_if1(SolverInstanceBase& si, const Call* call) {
     val1 = gi.exprToConst(call->arg(0));
     rhs -= val1;
   }
-  if (call->arg(1)->isa<Id>()) {
+  if (Expression::isa<Id>(call->arg(1))) {
     var2 = gi.exprToVar(call->arg(1));
     coefs.push_back(-1.0);
     vars.push_back(var2);
@@ -933,7 +940,7 @@ void p_indicator_eq_if1(SolverInstanceBase& si, const Call* call) {
     val2 = gi.exprToConst(call->arg(1));
     rhs += val2;
   }
-  if (call->arg(2)->isa<Id>()) {
+  if (Expression::isa<Id>(call->arg(2))) {
     varB = gi.exprToVar(call->arg(2));
   } else {
     fBconst = 1;
@@ -987,7 +994,7 @@ void p_cumulative(SolverInstanceBase& si, const Call* call) {
   double b = gi.exprToConst(call->arg(3));
 
   gi.getMIPWrapper()->addCumulative(
-      startTimes.size(), startTimes.data(), durations.data(), demands.data(), b,
+      static_cast<int>(startTimes.size()), startTimes.data(), durations.data(), demands.data(), b,
       make_constraint_name("p_cumulative_", (gi.getMIPWrapper()->nAddedRows++), call));
 }
 
@@ -1005,7 +1012,7 @@ void p_lex_lesseq_binary(SolverInstanceBase& si, const Call* call) {
   MZN_ASSERT_HARD(vec1.size() == vec2.size());
 
   gi.getMIPWrapper()->addLexLesseq(
-      vec1.size(), vec1.data(), vec2.data(), (bool)isModelCons,
+      static_cast<int>(vec1.size()), vec1.data(), vec2.data(), (bool)isModelCons,
       make_constraint_name("p_lex_lesseq__orbisack_", (gi.getMIPWrapper()->nAddedRows++), call));
 }
 
@@ -1023,7 +1030,9 @@ void p_lex_chain_lesseq_binary(SolverInstanceBase& si, const Call* call) {
   auto isModelCons = gi.exprToConst(call->arg(4));
 
   gi.getMIPWrapper()->addLexChainLesseq(
-      m, vars.size() / m, vars.data(), orbitopeType, (bool)resolveprop, (bool)isModelCons,
+      static_cast<int>(m), static_cast<int>(static_cast<double>(vars.size()) / m), vars.data(),
+      static_cast<int>(orbitopeType), static_cast<bool>(resolveprop),
+      static_cast<bool>(isModelCons),
       make_constraint_name("p_lex_lesseq__orbisack_", (gi.getMIPWrapper()->nAddedRows++), call));
 }
 
@@ -1043,7 +1052,7 @@ void p_xbz_cutgen(SolverInstanceBase& si, const Call* call) {
   //     cout << "  NEXT_CUTGEN" << endl;
   //     pCG->print( cout );
 
-  gi.registerCutGenerator(move(pCG));
+  gi.registerCutGenerator(std::move(pCG));
 }
 
 /// Initialize the SEC cut generator
@@ -1063,7 +1072,7 @@ void p_sec_cutgen(SolverInstanceBase& si, const Call* call) {
   //     cout << "  NEXT_CUTGEN" << endl;
   //     pCG->print( cout );
 
-  gi.registerCutGenerator(move(pCG));
+  gi.registerCutGenerator(std::move(pCG));
 }
 
 /// SCIP's bound disj
@@ -1085,8 +1094,8 @@ void p_bounds_disj(SolverInstanceBase& si, const Call* call) {
   gi.exprToVarArray(call->arg(5), varsF);
   double coef = 1.0;
   gi.getMIPWrapper()->addBoundsDisj(
-      fUB.size(), fUB.data(), bnd.data(), vars.data(), fUBF.size(), fUBF.data(), bndF.data(),
-      varsF.data(),
+      static_cast<int>(fUB.size()), fUB.data(), bnd.data(), vars.data(),
+      static_cast<int>(fUBF.size()), fUBF.data(), bndF.data(), varsF.data(),
       make_constraint_name("p_bounds_disj_", (gi.getMIPWrapper()->nAddedRows++), call));
 }
 
@@ -1098,7 +1107,7 @@ void p_array_minimum(SolverInstanceBase& si, const Call* call) {
   std::vector<MIPSolver::Variable> args;
   gi.exprToVarArray(call->arg(1), args);
   gi.getMIPWrapper()->addMinimum(
-      res, args.size(), args.data(),
+      res, static_cast<int>(args.size()), args.data(),
       make_constraint_name("p_minimum_", (gi.getMIPWrapper()->nAddedRows++), call));
 }
 
