@@ -16,7 +16,7 @@ namespace MiniZinc {
 EE flatten_anon(EnvI& env, const Ctx& ctx, Expression* e, VarDecl* r, VarDecl* b) {
   CallStackItem _csi(env, e);
   EE ret;
-  auto* av = e->cast<AnonVar>();
+  auto* av = Expression::cast<AnonVar>(e);
   if (av->type().isbot()) {
     throw InternalError("type of anonymous variable could not be inferred");
   }
