@@ -25,7 +25,7 @@ namespace {
 bool is_completely_par(EnvI& env, FunctionI* fi, const std::vector<Type>& tv) {
   if (fi->e() != nullptr) {
     // This is not a builtin, so check parameters
-    for (int i = 0; i < fi->paramCount(); i++) {
+    for (unsigned int i = 0; i < fi->paramCount(); i++) {
       if (fi->param(i)->type().isvar() && !fi->param(i)->type().any()) {
         return false;
       }
@@ -223,7 +223,7 @@ bool rhs_contains_var_comp(EnvI& env, Expression* e) {
     void vArrayAccess(const ArrayAccess* /*aa*/) {}
     /// Visit array comprehension
     void vComprehension(const Comprehension* c) {
-      for (int i = 0; i < c->numberOfGenerators(); i++) {
+      for (unsigned int i = 0; i < c->numberOfGenerators(); i++) {
         const auto* g_in = c->in(i);
         if (g_in != nullptr) {
           const Type& ty_in = Expression::type(g_in);
@@ -441,12 +441,18 @@ Call* generate_show(EnvI& env, Expression* e, Expression* w, Expression* p, bool
 
   if (t.bt() == Type::BT_TUPLE) {
     auto* tt = env.getTupleType(t);
+    if (tt->size() == 2 && (*tt)[1].isunknown()) {
+      auto field_type = (*tt)[0];
+      auto* field_access = new FieldAccess(Location().introduce(), e, IntLit::a(1LL));
+      Expression::type(field_access, field_type);
+      return generate_show(env, field_access, w, p, show_dzn, is_json);
+    }
     std::vector<Expression*> shown_fields(tt->size() == 1 && !is_json ? 4 : tt->size() * 2 + 1,
                                           tt->size() == 1
                                               ? new StringLit(Location().introduce(), ",")
                                               : new StringLit(Location().introduce(), ", "));
     bool canUseBuiltin = true;
-    for (size_t i = 0; i < tt->size(); i++) {
+    for (unsigned int i = 0; i < tt->size(); i++) {
       auto field_type = (*tt)[i];
       auto* field_access =
           new FieldAccess(Location().introduce(), e, IntLit::a(static_cast<long long int>(i + 1)));
@@ -483,7 +489,7 @@ Call* generate_show(EnvI& env, Expression* e, Expression* w, Expression* p, bool
     auto* rt = env.getRecordType(t);
     std::vector<Expression*> shown_fields(rt->size() * 2 + 2);
     bool canUseBuiltin = true;
-    for (size_t i = 0; i < rt->size(); i++) {
+    for (unsigned int i = 0; i < rt->size(); i++) {
       auto field_name = rt->fieldName(i);
       auto field_type = (*rt)[i];
       auto* field_access =
@@ -759,7 +765,7 @@ public:
         case Expression::E_ITE: {
           ITE* ite = Expression::cast<ITE>(e);
           stack.push_back(ite->elseExpr());
-          for (int i = 0; i < ite->size(); i++) {
+          for (unsigned int i = 0; i < ite->size(); i++) {
             stack.push_back(ite->ifExpr(i));
             stack.push_back(ite->thenExpr(i));
           }
@@ -833,9 +839,10 @@ void output_vardecls(EnvI& env, Item* ci, Expression* e) {
         ClearAnnotations::run(nvi->e());
         nvi->e()->introduced(false);
         if (reallyFlat != nullptr) {
-          env.outputFlatVarOccurrences.addIndex(reallyFlat, static_cast<int>(env.output->size()));
+          env.outputFlatVarOccurrences.addIndex(reallyFlat,
+                                                static_cast<unsigned int>(env.output->size()));
         }
-        env.outputVarOccurrences.addIndex(nvi, static_cast<int>(env.output->size()));
+        env.outputVarOccurrences.addIndex(nvi, static_cast<unsigned int>(env.output->size()));
         env.outputVarOccurrences.add(nvi->e(), ci);
         env.output->addItem(nvi);
 
@@ -1252,7 +1259,7 @@ Expression* create_dzn_output(EnvI& e, bool includeObjective, bool includeOutput
 
               auto* toStringMin =
                   Call::a(Location().introduce(), toString,
-                          {IntLit::a(idxMin), e.constants.literalFalse, e.constants.literalFalse});
+                          {IntLit::a(idxMin), e.constants.literalTrue, e.constants.literalFalse});
               toStringMin->type(Type::parstring());
               FunctionI* toStringMin_fi = e.model->matchFn(e, toStringMin, false);
               toStringMin->decl(toStringMin_fi);
@@ -1263,7 +1270,7 @@ Expression* create_dzn_output(EnvI& e, bool includeObjective, bool includeOutput
 
               auto* toStringMax =
                   Call::a(Location().introduce(), toString,
-                          {IntLit::a(idxMax), e.constants.literalFalse, e.constants.literalFalse});
+                          {IntLit::a(idxMax), e.constants.literalTrue, e.constants.literalFalse});
               toStringMax->type(Type::parstring());
               FunctionI* toStringMax_fi = e.model->matchFn(e, toStringMax, false);
               toStringMax->decl(toStringMax_fi);
@@ -1514,7 +1521,7 @@ void process_toplevel_output_vars(EnvI& e) {
                      e.model->filename().endsWith(".mzc.mzn")) {}
 
     bool hasAddToOutput = false;
-    std::vector<std::pair<int, VarDecl*>> todo;
+    std::vector<std::pair<size_t, VarDecl*>> todo;
 
     void vVarDeclI(VarDeclI* vdi) {
       auto* vd = vdi->e();
@@ -1546,7 +1553,7 @@ void process_toplevel_output_vars(EnvI& e) {
     // Insert implicit output variables
     int inserted = 0;
     for (auto& it : ovv.todo) {
-      auto idx = it.first;
+      int idx = static_cast<int>(it.first);
       auto* vd = it.second;
       if (Expression::ann(vd).contains(e.constants.ann.no_output) || Expression::type(vd).isPar()) {
         continue;
@@ -1883,7 +1890,7 @@ void create_output(EnvI& e, FlatteningOptions::OutputMode outputMode, bool outpu
             }
             if ((reallyFlat != nullptr) && env.outputFlatVarOccurrences.find(reallyFlat) == -1) {
               env.outputFlatVarOccurrences.addIndex(reallyFlat,
-                                                    static_cast<int>(env.output->size()));
+                                                    static_cast<unsigned int>(env.output->size()));
             }
           }
         } else {
@@ -1896,7 +1903,7 @@ void create_output(EnvI& e, FlatteningOptions::OutputMode outputMode, bool outpu
           }
         }
         make_par(env, vdi_copy->e());
-        env.outputVarOccurrences.addIndex(vdi_copy, static_cast<int>(env.output->size()));
+        env.outputVarOccurrences.addIndex(vdi_copy, static_cast<unsigned int>(env.output->size()));
         CollectOccurrencesE ce(env, env.outputVarOccurrences, vdi_copy);
         top_down(ce, vdi_copy->e());
         env.output->addItem(vdi_copy);
@@ -2125,7 +2132,7 @@ void finalise_output(EnvI& e) {
             vd->type(vdt);
             vd->ti()->type(vdt);
           }
-          e.outputVarOccurrences.addIndex(item->cast<VarDeclI>(), static_cast<int>(i));
+          e.outputVarOccurrences.addIndex(item->cast<VarDeclI>(), i);
           CollectOccurrencesE ce(e, e.outputVarOccurrences, item);
           top_down(ce, vd);
         } break;
