@@ -43,9 +43,12 @@ int run(const std::string& exe, const std::vector<std::string>& args, bool jsonS
       // Interrupted, just terminate
       if (slv.getFlagVerbose()) {
         std::cerr << std::endl << "Interrupted." << std::endl;
+        std::cerr << "   Done (";
+        std::cerr << "overall time " << startTime.stoptime() << ")." << std::endl;
       }
       // Re-raise signal
       e.raise();
+      return static_cast<int>(!fSuccess);
     } catch (const InternalError& e) {
       if (slv.getFlagVerbose()) {
         std::cerr << std::endl;
@@ -105,7 +108,7 @@ int wmain(int argc, wchar_t* argv[], wchar_t* envp[]) {
   }
   auto exe = FileUtils::wide_to_utf8(argv[0]);
 
-#ifdef NDEBUG
+#if defined(NDEBUG) && !defined(__MINGW32__)
   // Lambda to prevent object unwinding not allowed with __try..__except
   return ([&]() {
     __try {
