@@ -110,7 +110,8 @@ bool MIPosicbcWrapper::Options::processOption(int& i, std::vector<std::string>&
   std::string buffer;
   if (cop.get("-i")) {
     flagIntermediate = true;
-  } else if (string(argv[i]) == "-f") {  // NOLINT: Allow repeated empty if
+  } else if (string(argv[i]) == "-f" ||
+             string(argv[i]) == "--free-search") {  // NOLINT: Allow repeated empty if
     //     std::cerr << "  Flag -f: ignoring fixed strategy anyway." << std::endl;
   } else if (cop.get("--writeModel", &buffer)) {
     sExportModel = MiniZinc::FileUtils::file_path(buffer, workingDir);
@@ -193,8 +194,8 @@ std::vector<MiniZinc::SolverConfig::ExtraFlag> MIPosicbcWrapper::getExtraFlags(
       param_default = std::to_string(param.intParameter(model));
     } else if (t <= 400) {
       auto allowed = param.definedKeywords();
-      if (allowed.size() == 2 && (allowed[0] == "on" && allowed[1] == "off" ||
-                                  allowed[0] == "off" && allowed[1] == "on")) {
+      if (allowed.size() == 2 && ((allowed[0] == "on" && allowed[1] == "off") ||
+                                  (allowed[0] == "off" && allowed[1] == "on"))) {
         param_type = MiniZinc::SolverConfig::ExtraFlag::FlagType::T_BOOL;
       } else {
         param_type = MiniZinc::SolverConfig::ExtraFlag::FlagType::T_STRING;
@@ -364,7 +365,7 @@ static int statusOfCbc = 0;
 static CoinSighandler_t saveSignal = static_cast<CoinSighandler_t>(nullptr);
 
 extern "C" {
-static void
+void
 #if defined(_MSC_VER)
     __cdecl
 #endif  // _MSC_VER
